using ShortFormASR.Common;
using Newtonsoft.Json.Linq;
using System.Net.WebSockets;
using System.Text;

namespace ShortFormASR

{
    class Program
    {
        const string API_KEY = "xxx";
        const string API_SECRET = "xxx";
        const string APP_ID = "xxx";
        const string HOST_URL = "ws://iat-api-sg.xf-yun.com/v2/iat";
        const string audioPath = "D:\\static\\sdk\\ja.pcm";
        const int FRAME_SIZE = 1280; //The audio size of each frame
        const int INTERVAL = 40; //Send audio interval(ms)

        static async Task Main(string[] args)
        {
            Authentication.HttpMethod method = Authentication.HttpMethod.GET;
            Authentication authObj = new Authentication(API_KEY, API_SECRET);
            string authUrl = authObj.GenerateAuthUrl(HOST_URL, method);
            Console.WriteLine("authUrl---" + authUrl);

            var client = new ClientWebSocket();
            await client.ConnectAsync(new Uri(authUrl), CancellationToken.None);

            // readerBuffer
            BinaryReader br = new BinaryReader(new FileStream(audioPath, FileMode.Open));
            byte[] currentFrame = br.ReadBytes(FRAME_SIZE);
            bool moreData = true;

            int status = 0;

            // send first frame
            string message = GenerateData(APP_ID, status, currentFrame);
            var buffer = Encoding.UTF8.GetBytes(message);
            var sendBuffer = new ArraySegment<byte>(buffer);
            await client.SendAsync(sendBuffer, WebSocketMessageType.Text, true, CancellationToken.None);

            while (moreData)
            {
                currentFrame = br.ReadBytes(FRAME_SIZE);
                if (currentFrame.Length != 0)
                {
                    status = 1;
                }
                else
                {
                    status = 2;
                    moreData = false;
                }

                // Intermediate sending and ending sending
                string message2 = GenerateData(APP_ID, status, currentFrame);
                var buffer2 = Encoding.UTF8.GetBytes(message2);
                var sendBuffer2 = new ArraySegment<byte>(buffer2);
                await client.SendAsync(sendBuffer2, WebSocketMessageType.Text, true, CancellationToken.None);
            }

            br.Close();
            Console.WriteLine($"Sent: {message}");

            while (true)
            {
                // onMessage
                var receivebuffer = new ArraySegment<byte>(new byte[10240]);
                var result = await client.ReceiveAsync(receivebuffer, CancellationToken.None);
                string serverMessage = Encoding.UTF8.GetString(receivebuffer.Array, 0, result.Count);
                Console.WriteLine($"Server message: {serverMessage}");

                var state = client.State;
                Console.WriteLine($"state: {state}");

                if (state == WebSocketState.CloseReceived)
                {
                    // onClose
                    Console.WriteLine("onClose---");
                    await Task.Delay(5000);
                    await client.CloseAsync(WebSocketCloseStatus.NormalClosure, "Closing", CancellationToken.None);
                    break;
                }
            }

        }

        private static string GenerateData(string appId, int status, byte[] audio)
        {
            Thread.Sleep(INTERVAL);
            if (status == 0)
            {
                JObject commonObj = new JObject {
            { "app_id", appId }
            };
                JObject businessObj = new JObject {
            { "language", "ja_jp" },
            { "accent", "mandarin" },
            { "domain", "iat" }
            };
                JObject dataObj = new JObject {
            { "status", (int)status },
            { "format", "audio/L16;rate=16000" },
            { "encoding", "raw" },
            { "audio", Convert.ToBase64String(audio) }
            };
                JObject rootObj = new JObject { { "common", commonObj }, { "business", businessObj }, { "data", dataObj } };
                return rootObj.ToString();
            }
            else if (status == 1)
            {
                JObject dataObj = new JObject {
            { "status", (int)status },
            { "format", "audio/L16;rate=16000" },
            { "encoding", "raw" },
            { "audio", Convert.ToBase64String(audio) }
            };
                JObject rootObj = new JObject { { "data", dataObj } };
                return rootObj.ToString();
            }
            else
            {
                JObject dataObj = new JObject {
            { "status", (int)status },
            { "format", "audio/L16;rate=16000" },
            { "encoding", "raw" },
            { "audio", "" }
            };
                JObject rootObj = new JObject { { "data", dataObj } };
                return rootObj.ToString();
            }
        }

    }
}